<?php

/**
 * @file
 * Waiting queue aggregate administration functionality.
 */

/**
 * Waiting queue aggregate administration form.
 */
function waiting_queue_aggregate_admin_form($form, $form_state) {


  // Get some STATS.
  $processes = db_select('waiting_queue_aggregate_stats', 's')
    ->fields('s', array('last_seen_time', 'start_time', 'items_processed', 'status'))
    // Get stats from the two minutes.
    ->condition('s.last_seen_time', REQUEST_TIME - 120, '>')
    ->orderBy('s.start_time', 'DESC')
    ->orderBy('s.process_id')
    ->range(0, 10)
    ->execute();

  $form['process'] = array(
    '#caption' => t('Processing Statistics'),
    '#theme' => 'table',
    '#header' => array(
      t('Started'),
      t('Status'),
      t('Last seen'),
      t('Items processed'),
    ),
    '#rows' => array(),
    '#empty' => t('No processes observed in the two minutes.'),
    '#prefix' => '<div id="waiting-queue-aggregate-statistics-wrapper">',
    '#suffix' => '</div>',
  );

  foreach ($processes as $process) {
    $lifetime = $process->last_seen_time - $process->start_time;
    $total_items = $process->items_processed;
    if ($lifetime > 0) {
      $avg_items = ' (' . number_format($total_items / $lifetime, 2) . ' items/sec)';
    }
    else {
      $avg_items = '';
    }
    $form['process']['#rows'][] = array(
      format_interval(REQUEST_TIME - $process->start_time) . ' ago',
      check_plain($process->status),
      format_interval(REQUEST_TIME - $process->last_seen_time) . ' ago',
      number_format($total_items) . $avg_items,
    );
  }

  $form['statistics_update'] = array(
    '#type' => 'submit',
    '#ajax' => array(
      'callback' => 'waiting_queue_aggregate_admin_form_statistics_replace',
      'wrapper' => 'waiting-queue-aggregate-statistics-wrapper',
      'method' => 'replace',
      'speed' => 'fast',
      'progress' => 'none',
    ),
    '#value' => t('Update statistics'),
    '#submit' => array(
      'waiting_queue_aggregate_admin_form_dummy_submit',
    ),
    '#attributes' => array(
      'class' => array(
        'js-waiting-queue-aggregate-statistics-update',
      )
    ),
    '#attached' => array(
      'js' => array(
        drupal_get_path('module', 'waiting_queue_aggregate') . '/js/admin_statistics.js',
      ),
    ),
  );

  $options = array();

  $cron_queues = module_invoke_all('cron_queue_info');
  drupal_alter('cron_queue_info', $cron_queues);
  foreach ($cron_queues as $queue_name => $subqueue_info) {
    if (isset($cron_queues[$queue_name]['worker callback'])) {
      $options[$queue_name]['queue'] = $queue_name;
      /** @var \DrupalQueueInterface $queue */
      $queue = DrupalQueue::get($queue_name);
      $options[$queue_name]['count'] = number_format($queue->numberOfItems());
    }
  }

  // Check for modules implementing hook_queue_info().
  $queues = module_invoke_all('queue_info');
  drupal_alter('queue_info', $queues);
  foreach ($queues as $queue_name => $subqueue_info) {
    if (isset($cron_queues[$queue_name]['callback'])) {
      $options[$queue_name]['queue'] = $queue_name;
      /** @var \DrupalQueueInterface $queue */
      $queue = DrupalQueue::get($queue_name);
      $options[$queue_name]['count'] = number_format($queue->numberOfItems());
    }
  }

  unset($options['waiting_queue_aggregate']);

  // Sort the selected options to the 'top'.
  $default_value = variable_get('waiting_queue_aggregate_enabled_queues', array());
  $sorted_options = array();

  foreach ($default_value as $value) {
    if (isset($options[$value])) {
      $sorted_options[$value] = $options[$value];
      unset($options[$value]);
    }
  }
  foreach ($options as $name => $value) {
    $sorted_options[$name] = $value;
  }

  $form['waiting_queue_aggregate_enabled_queues'] = array(
    '#type' => 'tableselect',
    '#header' => array(
      'queue' => t('Queue'),
      'count' => t('Number of items'),
    ),
    '#options' => $sorted_options,
    '#multiple' => TRUE,
    '#default_value' => drupal_map_assoc($default_value),
  );

  $form['array_filter'] = array(
    '#type' => 'value',
    '#value' => TRUE,
  );

  return system_settings_form($form);
}

function waiting_queue_aggregate_admin_form_dummy_submit($form, &$form_state) {
  $form_state['rebuild'] = TRUE;
}

function waiting_queue_aggregate_admin_form_statistics_replace($form, $form_state) {
  return $form['process'];
}
